@@ -1,26 +1,25 @@
-package HTML::FormWidgets::Hidden;
+# @(#)$Id: Hidden.pm 248 2010-06-17 15:49:31Z pjf $
 
-# @(#)$Id: Hidden.pm 184 2009-06-13 22:25:28Z pjf $
+package HTML::FormWidgets::Hidden;
 
 use strict;
 use warnings;
+use version; our $VERSION = qv( sprintf '0.6.%d', q$Rev: 248 $ =~ /\d+/gmx );
 use parent qw(HTML::FormWidgets);
 
-use version; our $VERSION = qv( sprintf '0.5.%d', q$Rev: 184 $ =~ /\d+/gmx );
-
-sub _init {
+sub init {
    my ($self, $args) = @_;
 
    $self->container( 0 );
    return;
 }
 
-sub _render {
+sub render_field {
    my ($self, $args) = @_;
 
    delete $args->{id};
    $args->{type}  = q(hidden);
-   $args->{value} = delete $args->{default};
+   $args->{value} = delete $args->{default} || q();
 
    return $self->hacc->input( $args );
 }
