@@ -1,16 +1,15 @@
-package HTML::FormWidgets::Image;
+# @(#)$Id: Image.pm 224 2010-02-12 18:44:46Z pjf $
 
-# @(#)$Id: Image.pm 184 2009-06-13 22:25:28Z pjf $
+package HTML::FormWidgets::Image;
 
 use strict;
 use warnings;
+use version; our $VERSION = qv( sprintf '0.6.%d', q$Rev: 224 $ =~ /\d+/gmx );
 use parent qw(HTML::FormWidgets);
 
-use version; our $VERSION = qv( sprintf '0.5.%d', q$Rev: 184 $ =~ /\d+/gmx );
-
 __PACKAGE__->mk_accessors( qw(fhelp) );
 
-sub _init {
+sub init {
    my ($self, $args) = @_;
 
    $self->fhelp(   q() );
@@ -18,7 +17,7 @@ sub _init {
    return;
 }
 
-sub _render {
+sub render_field {
    my ($self, $args) = @_;
 
    return $self->hacc->img( { alt   => $self->fhelp,
