@@ -1,16 +1,15 @@
-package HTML::FormWidgets::RadioGroup;
+# @(#)$Id: RadioGroup.pm 224 2010-02-12 18:44:46Z pjf $
 
-# @(#)$Id: RadioGroup.pm 184 2009-06-13 22:25:28Z pjf $
+package HTML::FormWidgets::RadioGroup;
 
 use strict;
 use warnings;
+use version; our $VERSION = qv( sprintf '0.6.%d', q$Rev: 224 $ =~ /\d+/gmx );
 use parent qw(HTML::FormWidgets);
 
-use version; our $VERSION = qv( sprintf '0.5.%d', q$Rev: 184 $ =~ /\d+/gmx );
-
 __PACKAGE__->mk_accessors( qw(columns labels values) );
 
-sub _init {
+sub init {
    my ($self, $args) = @_;
 
    $self->columns( undef );
@@ -19,13 +18,14 @@ sub _init {
    return;
 }
 
-sub _render {
+sub render_field {
    my ($self, $args)   = @_;
 
-   $args->{columns}  = $self->columns  if ($self->columns);
-   $args->{labels}   = $self->labels   if ($self->labels);
-   $args->{onchange} = $self->onchange if ($self->onchange);
-   $args->{values}   = $self->values;
+   $args->{label_class} = q(radio_group);
+   $args->{columns    } = $self->columns  if ($self->columns);
+   $args->{labels     } = $self->labels   if ($self->labels);
+   $args->{onchange   } = $self->onchange if ($self->onchange);
+   $args->{values     } = $self->values;
 
    return $self->hacc->radio_group( $args );
 }
